/*
 * movement.c
 *
 *  Created on: 10/12/2012
 *      Author: Cesar
 *
 *  This library contains the functions to control the movement of a 2-wheel robot chassis
 *  powered by two modified servo motors.
 *  The modification of the servos consists of replacing the feedback position potentiometers with fixed resistors.
 *  This way, a PWM 3-6Vpp 50 Hz signal with a pulse width of 1500ms will stop the motor;
 *  a pulse width in the 1000-1500 ms range will make it spin in one direction,
 *  and a pulse width in the 1500-2000 ms range will make it spin in the opposite direction.
 *
 */



#include "pwm.h"
#include "movement.h"
#include "wifly.h"
#include "printf_library.h"
#include "uart.h"
#include <stdlib.h>
#include <math.h>
#include "FreeRTOS.h"
#include <task.h>


/*
 * int movement_Forward ()
 * Function that sends PWM signals to both motors to spin in the "forward" direction
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */
int movement_Forward(){

	PWM_Stop();

	PWM_Set( CHANNEL_LEFT, 1100 );
	PWM_Set( CHANNEL_RIGHT, 1100 );
	PWM_Start();
	return 0;
}

/*
 * int movement_Back ()
 * Function that sends PWM signals to both motors to spin in the "reverse" direction
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */
int movement_Back(){


	PWM_Stop();

	PWM_Set( CHANNEL_LEFT, 1800 );

	PWM_Set( CHANNEL_RIGHT, 1800 );

	PWM_Start();

	return 0;
}

/*
 * int movement_stop ()
 * Function that sends PWM signals to both motors to stop moving
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */
int movement_stop(){


	PWM_Stop();

	PWM_Set( CHANNEL_LEFT, 1500 );

	PWM_Set( CHANNEL_RIGHT, 1500 );

	return 0;
}

/*
 * int movement_pause ()
 * Function that shuts down temporarily both pwm outputs
 * so that movement can be resumed later on
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */
int movement_pause(){


	PWM_Stop();

	return 0;
}
/*
 * int movement_start ()
 * Function that enables both pwm outputs
 * without altering their configuration
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int movement_start(){


	PWM_Start();

	return 0;
}

/*
 * int movement_turnRight ()
 * Function that makes the robot turn to the right
 * by stopping the right side motor and making the left side motor advance
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */


int movement_turnRight(){


	PWM_Stop();

	PWM_Set( CHANNEL_LEFT, 1200 );

	PWM_Set( CHANNEL_RIGHT, 1800 );//1500

	PWM_Start();

	return 0;
}

/*
 * int movement_turnLeft ()
 * Function that makes the robot turn to the left
 * by stopping the left side motor and making the right side motor advance
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int movement_turnLeft(){



	PWM_Stop();

	PWM_Set( CHANNEL_LEFT, 1800 );//1500

	PWM_Set( CHANNEL_RIGHT, 1200 );

	PWM_Start();

	return 0;
}

/*
 * int movement_ToMaxRSSI_1D()
 *
 * Function that searches for the spot with the best wireless coverage
 * as measured from the RSSI parameter of the wifly module
 * while making the robot move in 1 dimension
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */

int movement_ToMaxRSSI_1D(int step_by_step){

	int maxRSSI;

	int valor = wifly_getRSSIint(1);//rssi value at the start point
	int checkpoint= -1000;
	int valorFinal = -1000;

	maxRSSI=valor;

	float errorRate= (valorFinal - maxRSSI) / maxRSSI;

	if (step_by_step){
		int timestep=512;

		while (errorRate > 0.1 || errorRate < -0.1){


			printfString1(UART_DEBUG, "going forward");
			while (valor >= maxRSSI){
				movement_Forward();//we move forward as long as the RSSI value goes up. So if the target is behind the unit, it will stop rapidly.

				vTaskDelay(timestep);
				movement_stop();
				valor = wifly_getRSSIint(1);
				if(valor >= maxRSSI) maxRSSI=valor;
			}
			printfString1(UART_DEBUG, "stop");

			valor=wifly_getRSSIint(1);
			if (valor < maxRSSI){//we advanced too far or started in the wrong direction, so we go back
				maxRSSI = valor; //reset maximum
				printfString1(UART_DEBUG, "going back");

				while (valor >= maxRSSI){
					movement_Back();

					vTaskDelay(timestep);
					movement_stop();
					valor = wifly_getRSSIint(1);
					if(valor >= maxRSSI) maxRSSI=valor;
				}
				printfString1(UART_DEBUG, "stop");
			}
			valorFinal = wifly_getRSSIint(1);
			errorRate= (valorFinal - maxRSSI)/ maxRSSI;
			//since we always stop after we leave a mimimum, we make sure we are within a reasonable distance from it
		}

	}else{//continuous movement
		while (errorRate > 0.1 || errorRate < -0.1){
			movement_Forward();//we move forward as long as the RSSI value goes up. So if the target is behind the unit, it will stop rapidly.

			printfString1(UART_DEBUG, "going forward");
			while (valor >= maxRSSI){
				maxRSSI=valor;
				valor = wifly_getRSSIint(1);
			}
			movement_stop();
			printfString1(UART_DEBUG, "stop");

			checkpoint=wifly_getRSSIint(1);
			if (checkpoint < maxRSSI){//we advanced too far or started in the wrong direction, so we go back
				movement_Back();
				maxRSSI = checkpoint; //reset maximum
				printfString1(UART_DEBUG, "going back");
				valor = wifly_getRSSIint(1);
				while (valor >= maxRSSI){
					maxRSSI=valor;
					valor = wifly_getRSSIint(1);
				}
				movement_stop();
				printfString1(UART_DEBUG, "stop");
			}
			valorFinal = wifly_getRSSIint(1);
			errorRate= (valorFinal - maxRSSI)/ maxRSSI;
			//since we always stop after we leave a mimimum, we make sure we are within a reasonable distance from it
		}
	}
	printfString1(UART_DEBUG, "SEARCH FINISHED");
	return 0;
}

/*
 * int movement_ToMaxRSSI_2D()
 *
 * Function that searches for the spot with the best wireless coverage
 * as measured from the RSSI parameter of the wifly module
 * while making the robot move in 2 dimensions
 * First, we make the unit turn to find the best direction
 * And then we call the 1-D search function to find the target
 * For this strategy to work, the wire antenna of the wifly unit must be oriented parallel to the ground
 * and pointing 90 degrees to the left or right. As this kind of the antenna has a toroidal energy distribution pattern
 * it will receive maximum energy when the access point is straight ahead or behind the unit.
 * Input parameters: none
 * Output parameters: none
 * Returns:	0 if all goes well; -1 in case of error
 */




int movement_ToMaxRSSI_2D(){
/* 1- find direction */
/* 2- apply 1D algorithm */




	int maxRSSI=-200 ;
	int timestep=256;

	int valor = 0;
	int valorFinal = -1000;
	int j=0;


	float errorRate= (valorFinal - maxRSSI) / maxRSSI;
	while (errorRate > 0.1 || errorRate < -0.1){

		valor = wifly_getRSSIint(1);
		maxRSSI=valor;
		j=0;
		//first, we make the unit turn full circle in small steps to find the best RSSI value, so we won't get stuck in a local maximum
		printfString1(UART_DEBUG, "turning right: phase1");
		while (j <10){
			movement_turnRight();

			vTaskDelay(timestep);
			movement_stop();
			valor = wifly_getRSSIint(1);
			if(valor >= maxRSSI) maxRSSI=valor;
			j++;
		}
		j=0;
		//second, we turn again trying to get within a reasonable distance of the said maximum
		printfString1(UART_DEBUG, "turning left: phase2");

		while (j <20 && valor < maxRSSI ){
			movement_turnLeft();

			vTaskDelay(timestep/2);
			movement_stop();
			valor = wifly_getRSSIint(1);
			j++;
		}
		valorFinal=valor;
	    errorRate= (valorFinal - maxRSSI) / maxRSSI;
	    timestep=timestep/2;//if not enough precision is achieved, we retry the process with finer movements
	}

	printfString1(UART_DEBUG, "finding range");
	return movement_ToMaxRSSI_1D(1);//Finally, we apply the 1-D function to reach the access point
}

