/*
 * pwm.c
 *
 *  Created on: 13/12/2012
 *      Author: Cesar
 *
 *  This library controls the PWM outputs of the LPCXpresso board at a lower level to provide the movement library with
 *  the suitable control functions
 *
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "LPC17xx.h"
#include "pwm.h"

static int initialized=0;

/*
 * void PWM_Ini( void )
 * Function that performs the basic initialization of the PWM outputs
 * We will be using the PWM2 and PWM4 outputs to send the 50Hz pwm signals required by the servo motors
 * Double-edged pulses must be used to avoid the PWM outputs to interfere with each other.
 * Therefore, 4 match registers are used
 * Input parameters: none
 * Output parameters: none
 * Returns:	nothing
 */
void PWM_Ini( void )
{

  LPC_PINCON->PINSEL4 &= ~0x000000FF; //Clear P2.0~3 Pins
  LPC_PINCON->PINSEL4 |=  0x00000044;// Set pins 43 (P2.1) and 45 (P2.3) for PWM2 and PWM4

  LPC_PWM1->TCR = 0x00000002;   /* Counter Reset */
  LPC_PWM1->PR  = 0x18;        /* Count Frequency = Peripheral clock / (24+1) = [1 Mhz]. So, the count unit is 1 uSec*/
  LPC_PWM1->MCR = (1<<1);      /* Reset TC on Match 0 */
  LPC_PWM1->MR0 = 20000;  /* set PWM cycle 20000uSec: 50Hz */
  //sets each pin to the 1.5ms pulsewidth need to send a "stop" signal to the servo
  LPC_PWM1->MR1 = 5000;/* start point of the pulse on PWM2 */
  LPC_PWM1->MR2 = 6500;/* end point of the pulse on PWM2 */
  LPC_PWM1->MR3 = 10000;/* start point of the pulse on PWM4 */
  LPC_PWM1->MR4 = 11500;/* end point of the pulse on PWM4 */

  /* PWM latch 0 to 4 enabled */
  LPC_PWM1->LER = LER0_EN | LER1_EN | LER2_EN | LER3_EN | LER4_EN;

  initialized=1;
  return;
}
/*
 * void PWM_Start (void)
 * Function that enables the PWM2 and PWM4 outputs to run double-edged pulses.
 * PWM_Init must be executed before.
 * Input parameters: none
 * Output parameters: none
 * Returns:	nothing
 */
void PWM_Start (void){
	if ( initialized==0)PWM_Ini();

	LPC_PWM1->TCR = (1<<0)|(1<<3);        //counter enable, PWM enable
	LPC_PWM1->PCR = (1<<10)|(1<<12)|(1<<2)|(1<<4);//PWM2 and PWM4 output enable, both dual-edged;
}
/*
 * void PWM_Stop (void)
 * Function that disables the PWM2 and PWM4 outputs.
 * Input parameters: none
 * Output parameters: none
 * Returns:	nothing
 */

void PWM_Stop (void){
	if ( initialized==0)PWM_Ini();
	LPC_PWM1->PCR &= 0 << 10; // disable PWM output 2
}

/*
 * PWM_Test( int mr)
 * Function that performs a basic test of the PWM functionality sensing single-edged pulses.
 * using output PWM1 on pin 42 (P2.0)
 * Input parameters:
 * 		- mr: desired pulse width
 * Output parameters: none
 * Returns:	nothing
 */

void PWM_Test( int mr){
	 if ( initialized==0)PWM_Ini();

	 LPC_SC->PCONP |= (1 << 6);            //PWM on
	 LPC_PINCON->PINSEL4 &=~3;            //reset
	 LPC_PINCON->PINSEL4 |= 1;            //set PWM1.1 at P2.0
	 LPC_PWM1->TCR = 2;                    //counter reset
	 LPC_PWM1->PR = 24;                    //clock /4 / prescaler (= PR +1) = 1 µs
	 LPC_PWM1->MCR = (1<<1);            //reset on MR0
	 LPC_PWM1->MR0 = 20000;                //set PWM cycle 50Hz
	 LPC_PWM1->MR1 = mr;//999;                //set duty
	 LPC_PWM1->LER = (1<<0)|(1<<1);        //latch MR0 & MR1
	 LPC_PWM1->PCR = (1<<9);            //PWM1 output enable
	 LPC_PWM1->TCR = (1<<0)|(1<<3);        //counter enable, PWM enable
	 return;
}


/*
 * void PWM_Set( int channelNum, int pulseWidth )
 * Function to set one PWM channel to the desired pulse width
 * Input parameters:
 * 		- channelNum: 1 to set the left channel, 2 to set the right channel
 * 		- pulseWidth: desired pulse width
 * Output parameters: none
 * Returns:	nothing
 */

void PWM_Set( int channelNum, int pulseWidth )
{
	if ( initialized==0)PWM_Ini();

  // Set new pulse width to match register and enable latch
  switch ( channelNum )
  {
    case CHANNEL_LEFT :
    	//remember,  MR1 value is 5000 so real pulse width is MR2-MR1
      LPC_PWM1->MR2 = (uint32_t)pulseWidth+5000; // Set match register 2 value
      LPC_PWM1->LER |= ((uint32_t)((2<7) ? (1<<2) : 0)); // Enable latch 2
      break;
    case CHANNEL_RIGHT :
    	//remember,  MR3 value is 10000 so real pulse width is MR4-MR3
      LPC_PWM1->MR4 = (uint32_t)pulseWidth+10000; // Set match register 4 value
      LPC_PWM1->LER |= ((uint32_t)((4<7) ? (1<<4) : 0)); // Enable latch 4
      break;
  }
  return;
}

/*
 * void PWM_Set2Channels( int pulseWidthLeft, int pulseWidthRight )
 * Function to set both PWM channels simultaneously.
 * When using single-edged pulses this works much better than setting each output separately
 * Input parameters:
 * 		- pulseWidthLeft: desired pulse width for the left channel
 * 		- pulseWidth: desired pulse width for the right channel
 * Output parameters: none
 * Returns:	nothing
 */

void PWM_Set2Channels( int pulseWidthLeft, int pulseWidthRight )
{
	if ( initialized==0)PWM_Ini();

	*((volatile unsigned int *)(&(LPC_PWM1->MR0) + 2)) = (uint32_t)pulseWidthLeft+5000;
	*((volatile unsigned int *)(&(LPC_PWM1->MR4) + 0)) = (uint32_t)pulseWidthRight+10000;
  // Set new pulse width to match register and enable latch
    LPC_PWM1->LER |=  (1 << 2) | (1 << 4 );

  return;
}


